
from grokcore.view.meta.views import default_view_name
import grokcore.component
import grokcore.view
import martian

from zeam.form.composed.form import SubFormBase
from zeam.form.composed.interfaces import ISubForm
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


def set_form_prefix(subform, form, name):
    """Recursively set the form prefix (to be compatible with groups)
    """
    # We use __dict__.get not to look if prefix was set in a parent class.
    if not subform.__dict__.get('prefix'):
        if not form.prefix:
            set_form_prefix(
                form,
                grokcore.view.view.bind().get(form),
                grokcore.view.name.bind(
                    get_default=default_view_name).get(form))
        subform.prefix = '%s.%s' % (form.prefix, name)


class SubFormGrokker(martian.ClassGrokker):
    """Grokker to register sub forms.
    """
    martian.component(SubFormBase)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.view.view)
    martian.directive(grokcore.view.name, get_default=default_view_name)

    def grok(self, name, factory, module_info, **kw):
        factory.module_info = module_info
        return super(SubFormGrokker, self).grok(
            name, factory, module_info, **kw)

    def execute(self, factory, config, context, layer, view, name, **kw):
        set_form_prefix(factory, view, name)

        adapts = (context, view, layer)
        config.action(
            discriminator=('adapter', adapts, ISubForm, name),
            callable=grokcore.component.util.provideAdapter,
            args=(factory, adapts, ISubForm, name),
            )
        return True




from megrok import pagetemplate as pt
from grokcore.component.util import sort_components
from grokcore import component as grok
from zope import component

from zeam.form.base import form
from zeam.form.composed import interfaces
from zope.interface import implementer

pt.templatedir('default_templates')


class SubFormBase(object):
    """Base class to be applied on a FormCanvas to get a subform.
    """
    grok.baseclass()
    # Set prefix to None, so it's changed by the grokker
    label = u''
    description = u''
    prefix = None

    def __init__(self, context, parent, request):
        super(SubFormBase, self).__init__(context, request)
        self.parent = parent

    def available(self):
        return True

    def htmlId(self):
        return self.prefix.replace('.', '-')

    def getComposedForm(self):
        return self.parent.getComposedForm()


@implementer(interfaces.ISubFormGroup)
class SubFormGroupBase(object):
    """A group of subforms: they can be grouped inside a composed form.
    """

    def __init__(self, context, request):
        super(SubFormGroupBase, self).__init__(context, request)
        # retrieve subforms by adaptation
        subforms = map(lambda f: f[1], component.getAdapters(
                (self.context, self,  self.request), interfaces.ISubForm))
        # sort them
        self.allSubforms = sort_components(subforms)
        self.subforms = []

    def getSubForm(self, identifier):
        for form in self.subforms:
            if form.htmlId() == identifier:
                return form
        return None

    def getComposedForm(self):
        return self

    def htmlId(self):
        return self.prefix.replace('.', '-')

    def update(self):
        # Call update for all forms
        for subform in self.allSubforms:
            subform.update()

    def updateActions(self):
        # Set/run actions for all forms
        form, action, status = self, None, None
        for subform in self._getAvailableSubForms():
            form, action, status = subform.updateActions()
            if action is not None:
                break
        return form, action, status

    def updateWidgets(self):
        # Set the subforms to render
        self.subforms = self._getAvailableSubForms()
        # Set widgets for all forms
        for subform in self.subforms:
            subform.updateWidgets()

    def _getAvailableSubForms(self):
        # filter out unavailables ones
        return filter(lambda f: f.available(), self.allSubforms)


@implementer(interfaces.ISimpleSubForm)
class SubForm(SubFormBase, form.FormCanvas):
    """Form designed to be included in an another form (a
    ComposedForm).
    """
    grok.baseclass()


class SubFormTemplate(pt.PageTemplate):
    """Default template for a SubForm
    """
    pt.view(SubForm)


@implementer(interfaces.ISubFormGroup)
class SubFormGroup(SubFormBase, SubFormGroupBase, form.GrokViewSupport):
    """A group of subforms.
    """
    grok.baseclass()

    def available(self):
        return len(self._getAvailableSubForms()) != 0


class SubFormGroupTemplate(pt.PageTemplate):
    """Default template for a SubFormGroup.
    """
    pt.view(SubFormGroup)


@implementer(interfaces.IComposedForm)
class ComposedForm(SubFormGroupBase, form.Form):
    """A form which is composed of other forms (SubForm).
    """
    grok.baseclass()

    def updateForm(self):
        executed_form, action, status = SubFormGroupBase.updateActions(self)
        if action is None:
            form.Form.updateActions(self)
        SubFormGroupBase.updateWidgets(self)
        form.Form.updateWidgets(self)


class ComposedFormTemplate(pt.PageTemplate):
    """Default template for a ComposedForm.
    """
    pt.view(ComposedForm)




from zope import interface
from zope.publisher.interfaces.browser import IBrowserView

from zeam.form.base.interfaces import IForm, IFormCanvas, ISimpleFormCanvas
from zeam.form.base.interfaces import IZeamFormBaseAPI


class ISubForm(IFormCanvas, IBrowserView):
    """A form designed to be included in an another form.
    """
    parent = interface.Attribute("Parent form")

    def available():
        """Return true if the form is available and should be
        rendered.
        """

    def getComposedForm():
        """Return the associated composed form.
        """


class ISimpleSubForm(ISubForm, ISimpleFormCanvas):
    """A simple sub form.
    """


class ISubFormGroup(interface.Interface):
    """A group of subforms.
    """
    subforms = interface.Attribute("List of available subforms")
    allSubforms = interface.Attribute("List of all subforms")

    def getSubForm(identifier):
        """Return a subform based on its HTML identifier.
        """


class IComposedForm(ISubFormGroup, IForm):
    """A form which is composed of other forms.
    """


class IZeamFormComposedAPI(IZeamFormBaseAPI):
    """API exported by zeamf.form.composed.
    """
    ComposedForm = interface.Attribute(
        u"A form which can be compose of other forms")
    SubForm = interface.Attribute(
        u"A form included in a ComposedForm")
    SubFormGroup = interface.Attribute(
        u"A group of subforms that behave like one subform")

    view = interface.Attribute(
        u"Directive to select which ComposedForm a SubForm belongs to")
    order = interface.Attribute(
        u"Directive used to order SubForms between themselves")



"""
We will define a composed form with some un-ordered subforms.

Let's grok our example:

  >>> from zeam.form.composed.testing import grok
  >>> grok('zeam.form.composed.ftests.forms.implicitorder')

We can now lookup our form by the name of its class:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.composed.ftests.forms.implicitorder import Content
  >>> context = Content()

  >>> from zope import component
  >>> form = component.getMultiAdapter(
  ...     (context, request), name='form')
  >>> form
  <zeam.form.composed.ftests.forms.implicitorder.Form object at ...>
  >>> form.updateForm()

Our form have subforms, which are in order D, C, B, and A:

  >>> form.subforms
  [<zeam.form.composed.ftests.forms.implicitorder.ASubForm object at ...>,
   <zeam.form.composed.ftests.forms.implicitorder.BSubForm object at ...>,
   <zeam.form.composed.ftests.forms.implicitorder.CSubForm object at ...>,
   <zeam.form.composed.ftests.forms.implicitorder.DSubForm object at ...>]

"""

from zeam.form import composed

from grokcore import component as grok


class Content(grok.Context):
    pass


class Form(composed.ComposedForm):
    label = u"Main form"


class ASubForm(composed.SubForm):
    composed.view(Form)

    label = u"Sub Form A"


class BSubForm(composed.SubForm):
    composed.view(Form)

    label = u"Sub Form B"


class CSubForm(composed.SubForm):
    composed.view(Form)

    label = u"Sub Form C"

class DSubForm(composed.SubForm):
    composed.view(Form)

    label = u"Sub Form D"



"""
We will define a composed form with some ordered subforms.

Let's grok our example:

  >>> from zeam.form.composed.testing import grok
  >>> grok('zeam.form.composed.ftests.forms.explicitorder')

We can now lookup our form by the name of its class:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.composed.ftests.forms.explicitorder import Content
  >>> context = Content()

  >>> from zope import component
  >>> form = component.getMultiAdapter(
  ...     (context, request), name='form')
  >>> form
  <zeam.form.composed.ftests.forms.explicitorder.Form object at ...>
  >>> form.updateForm()

Our form have subforms, which are in order D, C, B, and A, because we
used the order directive:

  >>> form.subforms
  [<zeam.form.composed.ftests.forms.explicitorder.DSubForm object at ...>,
   <zeam.form.composed.ftests.forms.explicitorder.CSubForm object at ...>,
   <zeam.form.composed.ftests.forms.explicitorder.BSubForm object at ...>,
   <zeam.form.composed.ftests.forms.explicitorder.ASubForm object at ...>]

"""

from zeam.form import composed

from grokcore import component as grok


class Content(grok.Context):
    pass


class Form(composed.ComposedForm):
    label = u"Main form"


class ASubForm(composed.SubForm):
    composed.view(Form)
    composed.order(20)

    label = u"Sub Form A"


class CSubForm(composed.SubForm):
    composed.view(Form)
    composed.order(10)

    label = u"Sub Form C"


class DSubForm(composed.SubForm):
    composed.view(Form)
    composed.order(0)

    label = u"Sub Form D"


class BSubForm(composed.SubForm):
    composed.view(Form)
    composed.order(15)

    label = u"Sub Form B"





import zeam.form.composed

from zope.app.wsgi.testlayer import BrowserLayer
from zope.configuration.config import ConfigurationMachine
from grokcore.component import zcml
import zope.testbrowser.wsgi


class Layer(
        zope.testbrowser.wsgi.TestBrowserLayer,
        BrowserLayer):
    pass


FunctionalLayer = Layer(zeam.form.composed, allowTearDown=True)


def grok(module_name):
    config = ConfigurationMachine()
    zcml.do_grok(module_name, config)
    config.execute_actions()
